@@ -68,6 +68,8 @@ class ObjectDef(Definition):
         self.typecode = None
 	self.fields = []
         self.implements = []
+        self.class_init_func = None
+        self.has_new_constructor_api = False
 	for arg in get_valid_scheme_definitions(args):
 	    if arg[0] == 'in-module':
 		self.module = arg[1]
@@ -113,7 +115,9 @@ class InterfaceDef(Definition):
 	self.module = None
 	self.c_name = None
         self.typecode = None
+        self.vtable = None
 	self.fields = []
+        self.interface_info = None
 	for arg in get_valid_scheme_definitions(args):
 	    if arg[0] == 'in-module':
 		self.module = arg[1]
@@ -121,6 +125,10 @@ class InterfaceDef(Definition):
 		self.c_name = arg[1]
 	    elif arg[0] == 'gtype-id':
 		self.typecode = arg[1]
+	    elif arg[0] == 'vtable':
+		self.vtable = arg[1]
+        if self.vtable is None:
+            self.vtable = self.c_name + "Iface"
     def write_defs(self, fp=sys.stdout):
 	fp.write('(define-interface ' + self.name + '\n')
 	if self.module:
@@ -253,7 +261,7 @@ class PointerDef(Definition):
             fp.write('  )\n')
 	fp.write(')\n\n')
 
-class MethodDef(Definition):
+class MethodDefBase(Definition):
     def __init__(self, name, *args):
         dump = 0
 	self.name = name
@@ -302,10 +310,6 @@ class MethodDef(Definition):
 
         if self.caller_owns_return is None and self.ret is not None:
             self.guess_return_value_ownership()
-        for item in ('c_name', 'of_object'):
-            if self.__dict__[item] == None:
-                self.write_defs(sys.stderr)
-                raise RuntimeError, "definition missing required %s" % (item,)
             
     def merge(self, old, parmerge):
         self.caller_owns_return = old.caller_owns_return
@@ -320,8 +324,7 @@ class MethodDef(Definition):
 		if p2[1] == pname:
 		    self.params[i] = (ptype, pname, p2[2], p2[3])
 		    break
-    def write_defs(self, fp=sys.stdout):
-	fp.write('(define-method ' + self.name + '\n')
+    def _write_defs(self, fp=sys.stdout):
 	if self.of_object != (None, None):
 	    fp.write('  (of-object "' + self.of_object + '")\n')
 	if self.c_name:
@@ -346,6 +349,24 @@ class MethodDef(Definition):
 	    fp.write('  (varargs #t)\n')
 	fp.write(')\n\n')
 
+
+class MethodDef(MethodDefBase):
+    def __init__(self, name, *args):
+        MethodDefBase.__init__(self, name, *args)
+        for item in ('c_name', 'of_object'):
+            if self.__dict__[item] == None:
+                self.write_defs(sys.stderr)
+                raise RuntimeError, "definition missing required %s" % (item,)
+        
+    def write_defs(self, fp=sys.stdout):
+	fp.write('(define-method ' + self.name + '\n')
+        self._write_defs(fp)
+
+class VirtualDef(MethodDefBase):
+    def write_defs(self, fp=sys.stdout):
+	fp.write('(define-virtual ' + self.name + '\n')
+        self._write_defs(fp)
+
 class FunctionDef(Definition):
     def __init__(self, name, *args):
         dump = 0
