@@ -33,8 +33,17 @@ except ImportError:
 import gobject as _gobject
 
 # load the required modules:
-import keysyms
+import sys
+sys_path = sys.path[:]
 from _gtk import *
+
+# init_gtk calls PySys_SetArgv which calls sys.path.insert(0, ''),
+# which causes problems for pychecker, restore it if modified.
+if sys.path != sys_path:
+    sys.path = sys_path
+del sys, sys_path
+
+import keysyms
 import gdk # this is created by the _gtk import
 
 threads_init = gdk.threads_init
@@ -123,4 +132,7 @@ FALSE = _DeprecatedConstant(False, 'gtk.FALSE', 'False')
 	    
 _gobject = _GObjectWrapper('gtk._gobject')
 
+# Can't figure out how to deprecate gdk.Warning
+gdk.Warning = Warning
+
 del _Deprecated, _DeprecatedConstant, _GObjectWrapper, _module, 
